
# q-expansion of newform 9702.2.a.ck, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9702
weight = 2
poly_data = [-1, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[-1, 0], [0, 0], [-1, -1], [0, 0], [1, 0], [0, 0], [-3, 1], [1, -3], [-2, -2], [0, 2], [5, 1], [-2, 4], [-3, 1], [2, -2], [-3, -3], [6, 0], [-6, -2], [0, 0], [2, -6], [-2, -2], [9, -3], [-2, -6], [-1, -1], [-2, 2], [0, 0], [-12, 0], [-13, -1], [8, 4], [6, 0], [0, 6], [-2, 2], [9, 1], [12, -2], [-1, -5], [6, 0], [0, 0], [17, 1], [-12, 0], [14, -2], [6, -6], [10, 2], [-5, 3], [6, -2], [14, 4], [0, -10], [3, -9], [-10, -6], [9, -3], [-7, -7], [-13, 3], [18, 4], [0, 0], [-7, 5], [-2, 2], [18, -2], [14, 2], [-21, 3], [-2, -2], [26, 0], [-2, 8], [-17, 3], [-18, -6], [-1, 11], [7, 7], [-10, -2], [14, -8], [6, 6], [-18, 0], [0, -12], [14, 6], [2, -10], [-6, -2], [-15, -3], [12, 6], [-4, 0], [15, 7], [-2, -4], [13, 5], [4, -2], [-5, -1], [24, 4], [-14, -4], [18, -2], [8, 8], [-18, 6], [6, 6], [-24, -2], [18, 0], [0, 12], [16, 8], [-12, 8], [0, -16], [28, -4], [12, -8], [-6, -6], [-10, 14], [-9, -1], [10, 14], [31, 3], [8, -6], [34, -2], [24, 6], [9, 1], [26, -4], [-22, -2], [26, -6], [22, 2], [21, 1], [6, 14], [7, 3], [8, 0], [8, -2], [-6, 8], [8, 12], [-8, -8], [18, -8], [-30, 6], [7, -17], [6, -4], [4, 8], [19, 11], [26, -4], [-42, 2], [-18, -10], [22, 2], [26, 8], [14, 4], [27, 3], [-43, 1], [-4, -12], [30, -6], [8, -8], [12, 12], [14, -4], [3, -9], [-17, -13], [-3, 21], [-11, -15], [33, 9], [-18, 12], [7, 11], [6, 16], [-4, 12], [8, 20], [-5, 19], [23, -1], [16, -8], [27, -9], [30, -6], [22, -10], [30, 0], [0, -12], [12, 0], [-28, 4], [-2, 14], [18, -6], [0, 24], [-14, -10], [-23, -11], [-30, -10], [20, -8], [22, -4], [28, -12], [-48, -4], [-6, -8], [45, -3], [-4, 12], [-14, 18], [26, 0], [-33, -1], [12, -16], [13, -11], [-24, -16], [10, -12], [31, -5], [-28, 8], [6, 18], [26, -8], [18, -6], [-10, -2], [-16, -8], [13, 13], [-34, 12], [3, 15], [2, 2], [-33, 7], [28, 14], [-16, 20], [-34, -8], [6, -2], [-14, 10], [56, -4], [-22, -6], [9, -23], [12, -8], [-18, -6], [6, 0], [26, -8], [-35, 1], [18, -6], [-18, 0], [38, 6], [29, -3], [25, -11], [24, 12], [-9, 15], [-5, -9], [-36, 0], [14, -20], [-16, -12], [-38, -12], [-27, 13], [12, -12], [6, -14], [62, -2], [10, -22], [18, -18], [-45, -9], [-42, -10], [10, 8], [4, -22], [-10, 6], [-12, 10], [-44, -4], [-15, 17], [38, 0], [-24, -12], [6, -18], [22, 14], [-12, 8], [-54, 0], [9, 21], [44, 12], [10, 8], [32, 4], [18, 10], [-68, -4], [6, 16], [-24, -20], [-3, -11], [-30, -14], [29, -15], [-25, -5], [-14, 12], [-29, -1], [26, -14], [11, -1], [-42, -6], [-62, 6], [22, -14], [12, 30], [-15, 5], [-42, 14], [-12, 12], [38, 10], [-24, -12], [22, 12], [-41, -13], [-33, 15], [-11, 9], [-16, 8], [-24, -12], [20, 4], [34, 18], [-52, -8], [42, 6], [-34, 8], [-8, 4], [-22, 6], [-68, -2], [37, 5], [44, 0], [-50, 6], [-22, 14], [13, 25], [-19, -15], [-12, 16], [-8, 10], [26, -8], [-45, 11], [8, 16], [-32, -8], [-20, 28], [22, 6], [9, 21], [-10, 14], [48, -6], [22, 16], [27, -9], [-25, 19], [42, -4], [36, -16], [-54, -12], [31, 7], [10, -24], [0, -12], [15, 3], [-9, 31], [21, -19], [-21, -9], [-9, 27], [-30, 0], [-22, -10], [34, 2], [-40, 20], [-36, -6], [-24, 12], [7, 23], [-30, 10], [-14, -8], [0, 16], [-18, -28], [18, -20], [-58, -10], [20, -28], [-19, 1], [18, -26], [-14, 14], [-26, -10], [-42, 0], [85, 1], [-18, 24], [-6, 14], [-8, 24], [-66, 4], [74, -6], [-28, -16], [-20, -16], [2, -10], [34, -8], [44, -2], [-10, 20], [-26, 6], [65, 1], [38, 10], [36, -24], [-66, -12], [63, -13], [28, -12], [-32, -16], [-6, -12], [2, -16], [66, 6], [4, 20], [50, -8], [-28, -16], [35, 3], [-64, 12], [-28, -20], [-63, 1], [47, 11], [-4, -10], [-30, -12], [-16, 16], [66, 0], [23, 19], [1, -23], [-31, -7], [-18, 28], [38, -2], [2, 32], [-33, 27], [6, 6], [16, 4], [12, -24], [-42, 0], [42, 22], [10, -18], [38, -12], [-44, 16], [20, -24], [6, 26], [8, 14], [-40, 24], [14, 24], [-48, 12], [-38, -2], [-13, -1], [-15, 5], [1, 21], [-43, 13], [-66, 0], [82, -2], [-18, -20], [-65, -13], [51, 3], [-36, 12], [-21, 27], [-34, 18], [25, 1], [-26, 24], [21, -19], [16, 20], [42, -18], [46, -2], [34, 20], [-28, 12], [36, 8], [-10, -2], [-22, -32], [12, -36], [6, 8], [-20, -20], [-24, 12], [50, 22], [-22, -22], [30, 0], [-14, -8], [16, 4], [44, 16], [-7, -27], [40, -14], [-36, 8], [-2, -14], [2, -20], [-27, 5], [6, -22], [-27, 33], [48, -20], [48, -16], [-6, 40], [28, 28], [2, 2], [15, 15], [-2, 6], [-3, 21], [-6, 2], [-26, 32], [48, -12], [4, 4], [22, 22], [52, -8], [26, -14], [-21, 35], [-37, -13], [-34, -22], [-2, 8], [-34, -12], [-8, 4], [-50, 6], [38, -22], [-6, 0], [-68, 2], [-36, 20], [-27, 9], [18, 24], [-35, 9], [54, 6], [-28, -20], [-68, -4], [30, 18], [45, -3], [8, -4], [9, -11], [18, 0], [-6, -20], [8, 22], [-5, 11], [-56, 0], [8, -10], [6, 6], [-24, -24], [-48, -28], [-58, 16], [59, 7], [26, 12], [40, 0], [-14, 14], [30, -24], [54, -6], [-76, -12], [-20, 16], [-39, -15], [-66, -12], [72, -12], [-18, -18], [6, -28], [-15, 21], [80, 8], [-54, 4], [-42, -18], [47, 11], [-42, 6], [18, -10], [-79, -11], [12, -8], [-26, -36], [-58, 2], [-84, 0], [-16, 8], [-10, -18], [-14, 10], [-52, 22], [24, 28], [4, 32], [-44, -12], [-16, -12], [-46, 28], [-6, -6], [10, 30], [-22, -8], [0, 24], [54, 12], [2, 14], [-6, 12], [-48, -12], [-14, -22], [5, 33], [40, -18], [1, 5], [-21, 11], [-48, -8], [61, 17], [10, 28], [-38, -2], [-12, 6], [13, 13], [24, 0], [2, 26], [27, 15], [14, -28], [-46, 30], [34, -8], [54, 6], [-95, -3], [27, -5], [44, 4], [16, 4], [12, -16], [-37, 11], [-18, -30], [42, 32], [-94, -2], [-70, -14], [61, 21], [-10, -2], [15, -33], [-36, 32], [2, 12], [74, 6], [-6, -6], [66, -12], [51, -9], [-63, 17], [66, -16], [16, -28], [44, 4], [-28, 16], [-20, 36], [22, 10], [-42, 12], [73, 13], [-11, -35], [-6, -24], [-94, 2], [58, 18], [70, 14]]
